#
# Christmas Logic 
# Copyright (C) Jack Whitham 2009
#

from baselevel import *

(one, two, three) = Gen_Flakes(Create_String, 3)

one.Add_Sequence("1000")
two.Add_Sequence("0100")
three.Add_Sequence("0010")

Create_Component(Light_Control, one)
Create_Component(Light_Control, two)
Create_Component(Light_Control, three)

start = Create_Component(Stimulator, "start", "1000")
power = Create_Component(Stimulator_Simple, ST_ONE)

delays = [ Create_Component(D_Flip_Flop),
    Create_Component(D_Flip_Flop) ]
locations = set([ c.Get_Location() for c in delays ])



Meta_Data(text = "Snow is Falling Again",
    desc = "Light the snowflakes, again! The sequence should be: top light, "
        "middle light, bottom light, then nothing.\n\n"
        "This time, however, you don't get 'Or', 'And', or 'Not' components. "
        "Instead, you get the new 'Delay' component. The 'Delay' component "
        "has two inputs: data ('d') and clock enable ('ce'). When 'ce' is "
        "'On', Delay remembers the value of 'd' and produces it on the output "
        "during the next cycle. That's different to all the other components "
        "you've seen so far, because those produce an output immediately. "
        "When 'ce' is 'Off', Delay keeps its current value.",
    guide = [
    Guide(trigger = "True",
        point_at = "delays[ 1 ]",
        text = "This is a 'Delay' component. The topmost input is for data. "
            "The other input is 'ce': clock enable. The Delay component "
            "remembers the last data input it received while 'ce' was 'On'. "
            "While 'ce' is 'Off', the output of 'Delay' does not change. "
            "Click 'Next...' to go on.",
        requires_ack = True),
    Guide(trigger = "True",
        point_at = "start.Get_Terminal()",
        text = "Connect the 'start' input to the 'data' input of one 'Delay' "
            "component."),
    Guide(trigger = "Mix_Tool(delays, [start], 1)",
        point_at = "power.Get_Terminal()",
        text = "Connect the 'power' input to the 'ce' input of both 'Delay' "
            "components."),
    Guide(trigger = "Mix_Tool(delays, [start], 1) and "
                "Mix_Tool(delays, [power], 2)",
        point_at = "None",
        text = "Connect the output of one 'Delay' component to the other."),
    Guide(trigger = "Mix_Tool(delays, [start], 1) and "
                "Mix_Tool(delays, [power], 2) and "
                "Mix_Tool(delays, delays, 1)",
        point_at = "None",
        text = "Notice that the signal from 'start' is propagating "
            "through the wiring! Now complete the level by connecting "
            "the lights."),

    ])

